# Hapi API Server with Mol And Sol
Ok so last post we've seen how to create controller routes to our Hapi server
now lets see how we can use Mol to add models as well and connect them to our controllers;

To makes things fun we will also use another Hapi Plugin called Sol to manage session auth to our server
so lets create a basic user model
```javascript
var hoek    =  require('hoek'),
	STORAGE = [
		{id: 1, name: 'alice', password: 'password'},
		{id: 2, name: 'bob',   password: 'password1'}
	];


module.exports = {
	findBy: function userFindBy (param, value) {
		return  STORAGE.reduce(function (res, item) {
			return res || (item[param] === value && hoek.clone(item));
		}, false);
	}
};
```

Nothing fancy here either. we have a very basic model that has some fixed users and the only method is `findBy` which gets a `param` name and a `value` to search, so we can call `model.findBy('name', 'bob')` and get back bob's record.

Now we'll need to create a login controller.
this controller is a bit more complex than what we had before, So I wont go much into the login/logout logic which I just copied form Hapi Sol auth plugin (check Hapi Sol Github repo for more info about how to work with it.
```javascript
var scope;

function setScope (newScope) {
	scope = newScope;
}

function getModel() {
	return scope.models.user;
}
module.exports = [{
	method: ['POST', 'GET'],
	path: '/login',
	config: {
		handler: function (request, reply) {
			var message = '',
				account = null;

			if (request.auth.isAuthenticated) {
				return reply.redirect('/');
			}

			if (request.method === 'post') {

				if (!request.payload.username || !request.payload.password) {
					message = 'Missing username or password';
				} else {
					account = getModel().findBy('name', request.payload.username);
					if (!account || account.password !== request.payload.password) {
						message = 'Invalid username or password';
					}
				}
			}

			if (request.method === 'get' || message) {
				return reply('<html><head><title>Login page</title></head><body>' + (message ? '<h3>' + message + '</h3><br/>' : '') + '<form method="post" action="/login">' + 'Username: <input type="text" name="username"><br>' + 'Password: <input type="password" name="password"><br/>' + '<input type="submit" value="Login"></form></body></html>');
			} else {
				return request.auth.session.set(account, function () {
					return reply.redirect('/');
				});
			}
		},
		auth: {
			mode: 'try'
		},
		plugins: {
			'sol': {
				redirectTo: false
			}
		}
	}
}, {
	method : 'GET',
	path   : '/logout',
	handler: function (request, reply) {
		return request.auth.session.clear(function () {
			return reply.redirect('/');
		});
	}
}];

module.exports.setScope = setScope;
```

The main thing to notice is that we augmented our controller structure.
Instead of just exporting an array of routes config we are exporting the methods `setScope` and `getModel` on top of the routes Array, not the cleanest way to do it but for this example it will do.

On line 27 we use the `getModel` method to get the user model and find the account of the user being signed in. All we need is to make sure we set the scope after loading the controller and were good to go.
```javascript
var hapi   = require('hapi'),
	mol   = require('../'),
	sol    = require('../'),
	server = new hapi.Server();


server.connection({port: 8000});

server.register([sol, mol], function (err){
	if (err) {
		throw err;
	}
	var App = {
		models: {},
		controllers: {}
	};

	server.auth.strategy('session', 'session', true, {
		redirectTo: '/login',
		isSecure: false
	});

	server.plugins.mol.load('models', App.models, function (path, base, ext, scope){
		scope[base] = require(path);
	}).then(function (){
		server.plugins.mol.load('controllers', App.controllers, function (path, base, ext, scope){
			var controller = require(path);
			scope[base] = controller;
			if (controller.setScope) {
				controller.setScope(App);
			}
			server.route(controller);
		});
	}).then(server.start.bind(server, function (err) {
		if (err) {
			console.error(err);
		} else {
			console.log('Server running at localhost:8000');
		}
	}));
});
```

As before I wont go into Sol stuff and just look at Mol specific logic.
As you can see we first tell Mol to load all of our models into `App.models` scope
then we load all our controllers and after setting each controller to `App.controllers` scope
we call the `setScope` method on the controller setting `App` as the scope and register the controller route with our server.

Cool but didn't we say we want to have an api server ?
So lets add another controller called user under `controllers/api/`
this controller will let authenticated users to fetch their own record

```javascript
var scope;

function setScope (newScope) {
	scope = newScope;
}

function getModel() {
	return scope.models.user;
}

module.exports = [{
	method: 'GET',
	path: '/me',
	handler: function (request, reply) {
		var user = getModel().findBy('id', request.auth.credentials.id);
		user.password = null;
		reply(user);
	}
}];

module.exports.setScope = setScope;
```

So we are now adding the route `/me` to our app that will fetch a user record according to our credentials and send it as a reply (we also set the password to null since there's no reason to send it back in the api).

So is that it ?
Almost. I guess we can leave it this way, but notice that now our routes are kinda messy.
this is not a problem in such a small example but if we had a large api we probably want `/user/1` to return user with id 1 and `/post/1` to return the first post and we probably want all this routes to be prefixed with `/api` so they only return data and similar routes might return a full html page for search bots and users who don't use js.
we can hard code this specific routes in our controller code, but if we ever want to change them its going to be a mess, and we can accidentally create collisions and typos in our routes.

So lets see how we can solve this using Mol when loading the controllers without hard coding our routes -
```javascript
server.plugins.colm.load('controllers', App.controllers, function (path, base, ext, scope, parts){
	var prefix,
		controller = require(path);
	scope[base] = controller;
	if (controller.setScope) {
		controller.setScope(App);
	}
	if (parts[0] === 'api') {
		prefix = parts.concat(base).join('/');
		controller.forEach(function (route){
			route.path = '/' + prefix + route.path;
		});
	}
	server.route(controller);
});
```
The last parameter Mol sends to our register callback `parts` is an array with the subfolders of the file. so if the file is in the `api` folder, we prefix the path for each route with the folder and the base name, so all the routes for our user api will start with `api/user`

if we ever need to change the prefixed routes we can do it in our loader and we don't have to change individual files.

Next post we'll see how to make our code easier to maintain and DRY using inheritance.












before we see how we can use Mol to connect the controller with the model we first need to make sure our controller has an awarness to the model concept so lets create a user controller with a few small changes to what we had with the hello controller -








this function accepts the following parameters
`loadPath`  - path to load files from
`scope` (optional) - the scope object which will be passed to the register callback if `flat` is `false` scope will be relative to subfolders so when calling the register callback for 'base_path/foo/bar.js' scope will be scope['foo']
`flat` (optional) - if `true` scope will `registerCb` scope will always be the same default(false)
`registerCb`  - the callback that will be called for each file in `loadPath`
`cb  (optional) - callback to call once all files are loaded, since Mol uses promises you can leave it blank and just follow the promise chain




